import json
import os, sys
import argparse

import requests
from requests.auth import HTTPBasicAuth


class BitBucketException(Exception):
    pass


class BitBucket:
    def __init__(self, user_name, password, team):
        self.base_url = 'https://api.bitbucket.org/2.0'
        self.team = team
        self.auth = HTTPBasicAuth(user_name, password)

    def if_branch_in_prs(self, project_name, branch_name):
        # filter for branch name
        opened_prs = self._get_opened_pr(project_name)
        source_branches = [ p.get('source').get('branch').get('name') for p in opened_prs]
        if branch_name in source_branches:
            return True

    def get_last_commit_author(self, repo_name, branch_name):
        commit_hash = self._get_branch_info(repo_name, branch_name)['target']['hash']
        return self._get_commit_info(repo_name, commit_hash)['author']['raw']

    def start_custom_pipeline(self, repo_name, build_branch, pipeline_name, variables):
        data = {
            'target': {
                "type": "pipeline_ref_target",
                "ref_type": "branch",
                "ref_name": build_branch,
                "selector": {
                    "type": "custom",
                    "pattern": pipeline_name
                }
            },
        }
        if variables:
            data.update({'variables': variables})
        print(data)
        url = '{}/repositories/{}/{}/pipelines/'.format(self.base_url, self.team, repo_name)
        req = requests.post(url, auth=self.auth, json=data)
        print(req)
        print(req.text)

    def _get_opened_pr(self, project):
        url = f'{self.base_url}/repositories/{self.team}/{project}/pullrequests?q=state="open"'
        resp = requests.get(url, auth=self.auth)
        if resp.status_code == 200:
            return resp.json()['values']

    def _get_branch_info(self, repo_name, branch_name):
        url = '{}/repositories/{}/{}/refs/branches/{}'.format(
            self.base_url, self.team,
            repo_name, branch_name
        )
        req = requests.get(url, auth=self.auth)
        return req.json()

    def _get_commit_info(self, repo_name, commit_hash):
        url = '{}/repositories/{}/{}/commit/{}'.format(
            self.base_url, self.team,
            repo_name, commit_hash
        )
        req = requests.get(url, auth=self.auth)
        return req.json()


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--repo", type=str, required=True)
    parser.add_argument("--branch", type=str, default="master")
    parser.add_argument("--pipeline", type=str, required=True)
    parser.add_argument("--pipe-vars", type=str)
    return parser.parse_args()


def main():
    bb = BitBucket(os.getenv('BB_CLIENT'), os.getenv('BB_TOKEN'), 'neurohive')
    args = parse_args()
    pipe_vars = None
    if args.pipe_vars:
        pipe_vars = json.loads(args.pipe_vars)
    bb.start_custom_pipeline(args.repo, args.branch, args.pipeline, pipe_vars)


def check_branch():
    parser = argparse.ArgumentParser()
    parser.add_argument('--branch', type=str, required=True)
    parser.add_argument('--project-name', type=str, required=True)
    args = parser.parse_args()
    bb = BitBucket(os.environ['BB_CLIENT'], os.environ['BB_TOKEN'], 'neurohive')
    if bb.if_branch_in_prs(args.project_name, args.branch):
        print("found in pull requests")
        sys.exit()
    else:
        print("not in pull requests")
        sys.exit(1)


if __name__ == '__main__':
    pass



APPSPEC = {
    "version": 0.0,
    "Resources": [
        {
            "TargetService": {
                "Type": "AWS::ECS::Service",
                "Properties": {
                    "TaskDefinition": "ПОМЕНЯТЬ",
                    "LoadBalancerInfo": {
                        "ContainerName": "ПОМЕНЯТЬ",
                        "ContainerPort": "ПОМЕНЯТЬ"
                    },
                    "PlatformVersion": "LATEST"
                }
            }
        }
    ]
}



import os, sys
import argparse
import json
import re
from datetime import datetime
import logging

from neurohive.integration.bitbucket import BitBucket
from neurohive.integration.unitycloud import UnityCloud
from neurohive.integration.appcenter import AppCenter
from neurohive.deploy.ecs import AwsEcsDeploy

def get_jira_name_from_branch(branch_name):
    regexp = '(feature|bugfix)?(?P<name>\w+-\d+)'
    match = re.search(regexp, branch_name)
    if match:
        return match.groupdict()['name']


def create_uc_builds_from_branches(uc_client, branches, src_build_target):
    now = datetime.now().strftime('%Y-%m-%d-%H%M')
    to_build = []
    for branch in branches:
        jira_name = get_jira_name_from_branch(branch)
        name = f'AutoPR-{now}--{branch}'
        if jira_name:
            name = f'AutoPR-{now}--{jira_name}'
        to_build.append({
            "branch": branch,
            "target_name": name
        })
    targets_data = []
    for build in to_build:
        logging.info(f'Create build: {build}')
        targets_data.append(uc_client.clone_build_target(src_build_target, build['target_name'], build['branch']))
    return targets_data


def create_and_build_from_prs():
    parser = argparse.ArgumentParser()
    parser.add_argument("--git-project", type=str, required=True)
    parser.add_argument("--unity-project", type=str, required=True)
    parser.add_argument("--src-build-target-name", type=str, required=True)
    args = parser.parse_args()

    bb = BitBucket(os.getenv('BB_CLIENT'), os.getenv('BB_TOKEN'), 'neurohive')
    uc = UnityCloud(os.getenv('UNITY_ORG_ID', 'unity_oekd8fheerjomw'), project_name=args.unity_project)

    # # удаляю прошлые сборки )
    logging.basicConfig(level=logging.INFO)
    uc.remove_target_with_prefix('AutoPR')
    #получаю список веток с открытими prs
    opened_prs = bb._get_opened_pr(args.git_project)
    branches = [branch['source']['branch']['name'] for branch in opened_prs]
    build_targets = create_uc_builds_from_branches(uc, branches, args.src_build_target_name)
    # запускаю
    for build_target in build_targets:
        uc._create_build(build_target['buildtargetid'])


def cleanup_old_appcenter_build():
    parser = argparse.ArgumentParser()
    parser.add_argument("--app-name", type=str, required=True)
    parser.add_argument("--owner", type=str, required=True)
    parser.add_argument("--leave-num", type=int, default=10)
    args = parser.parse_args()
    ac = AppCenter(args.owner)
    ac.cleanup_old_builds(args.app_name, args.leave_num)


def deploy_ecs_task():
    parser = argparse.ArgumentParser()
    parser.add_argument("--def-settings-dir", type=str, required=True)
    parser.add_argument("--codedeploy-settings", type=str, required=True)
    args = parser.parse_args()
    ecs = AwsEcsDeploy(args.def_settings_dir, args.codedeploy_settings)
    ecs.create_deploy()



import os
import subprocess
from subprocess import CalledProcessError
import logging
import json
from copy import deepcopy

import boto3

from neurohive.deploy.templates import APPSPEC

logger = logging.getLogger()


class AwsEcsDeployException(Exception):
    pass


class AwsEcsDeploy:
    def __init__(self, def_dir, cd_settings):
        self.def_dir = def_dir
        self.cd_settings = cd_settings
        self._check_reqs()

    def create_deploy(self):
        self.prep_task_from_settings_dir()
        appspec_string = self._prep_appspec()
        with open(self.cd_settings, 'rt') as fp:
            deploy = json.load(fp)
        deploy['revision']['appSpecContent']['content'] = appspec_string
        client = boto3.client('codedeploy')

        try:
            resp = client.create_deployment(**deploy)
            if 'deploymentId' not in resp:
                raise AwsEcsDeployException("Deployment failed")
        except Exception as e:
            logger.error(e)
            raise

    def _prep_appspec(self):
        task_def = self._get_terraform_output()['task_def']['value']
        appspec = deepcopy(APPSPEC)
        tg_props = appspec['Resources'][0]['TargetService']['Properties']
        tg_props['TaskDefinition'] = task_def['arn']
        tg_props['LoadBalancerInfo']['ContainerName'] = task_def['lb_continer_name']
        tg_props['LoadBalancerInfo']['ContainerPort'] = task_def['lb_container_port']
        return json.dumps(appspec)

    def prep_task_from_settings_dir(self):
        settings_path = os.path.abspath(self.def_dir)
        self._init_terraform(self.def_dir)
        self._apply_terraform(self.def_dir)

    def _get_terraform_output(self):
        cmd = ['terraform', 'output', '-no-color', '-json']
        try:
            output = json.loads(subprocess.run(cmd, cwd=self.def_dir, check=True, capture_output=True).stdout)
            return output
        except CalledProcessError as e:
            logger.error('terraform output failed')
            raise AwsEcsDeployException(e)

    def _init_terraform(self):
        try:
            cmd = ['terraform', 'init']
            subprocess.run(cmd, cwd=self.def_dir, check=True)
        except CalledProcessError as e:
            logger.error('terraform init failed')
            raise AwsEcsDeployException(e)

    def _apply_terraform(self):
        cmd = ['terraform', 'apply', '-auto-approve', '-no-color']
        try:
            subprocess.run(cmd, cwd=self.def_dir, check=True)
        except CalledProcessError as e:
            logger.error("failed to apply terraform changes")
            raise AwsEcsDeployException(e)

    def _check_reqs(self):
        try:
            os.environ['AWS_SECRET_ACCESS_KEY']
            os.environ['AWS_ACCESS_KEY_ID']
        except KeyError as e:
            logger.error("AWS Credentials required")
            raise AwsEcsDeployException(e)

        try:
            subprocess.run(['terraform', '-v'], check=True)
        except CalledProcessError as e:
            logger.error("terraform executin checks failed")
            raise AwsEcsDeployException(e)
        except FileNotFoundError:
            logger.error("terraform required")
            raise AwsEcsDeployException()

        if not os.path.isdir(self.def_dir):
            raise AwsEcsDeployException("Specify terraform task setup dir")
        if not os.path.isfile(self.cd_settings):
            raise AwsEcsDeployException("Specify codedeploy settings")



from datetime import datetime

import requests
from requests.exceptions import RequestException
import jwt


class AppStoreConnectException(Exception):
    pass


class AppStoreConnect:
    def __init__(self, key_id: str, issuer_id: str, private_key: bytes) -> None:
        self.key_id = key_id
        self.issuer_id = issuer_id
        self.private_key = private_key
        self.headers = {
            'alg': 'ES256',
            'kid': self.key_id,
            'typ': 'JWT'
        }
        self.base_url = 'https://api.appstoreconnect.apple.com/v1'

    def _prep_headers(self):
        payload = {
            'iss': self.issuer_id,
            'exp': int(datetime.now().strftime("%s")) + 60 * 5,
            'aud': 'appstoreconnect-v1'
        }
        token = jwt.encode(payload, self.private_key, algorithm='ES256', headers=self.headers)
        return {'Authorization': 'Bearer %s' % token.decode('utf-8')}

    def add_device(self, udid, name):
        url = '{}/devices'.format(self.base_url)
        data = {
            'data': {
                'type': 'devices',
                'attributes': {
                    'name': name,
                    'udid': udid,
                    'platform': 'IOS'
                }
            }
        }
        req = requests.post(url, headers=self._prep_headers(), json=data)
        if req.status_code != 201:
            raise AppStoreConnectException(req.status_code)

    def list_devices(self):
        url = '{}/devices'.format(self.base_url)
        req = requests.get(url, headers=self._prep_headers())
        print(req.status_code)
        print(req.json())

    def get_device_info(self, dev_id):
        url = '{}/devices/{}'.format(self.base_url, dev_id)
        req = requests.get(url, headers=self._prep_headers())
        print(req.text)

    def apps(self):
        url = '{}/apps'.format(self.base_url)
        print(url)
        print(self._prep_headers())
        try:
            req = requests.get(url, headers=self._prep_headers())
            if req.status_code != 200:

                raise AppStoreConnectException
        except RequestException as e:
            raise AppStoreConnectException(e)


if __name__ == '__main__':
    pass
